"""UW Identity Provider Configuration."""
from . import IdpConfig, attribute


class UwIdp(IdpConfig):
    """UW's IdP. SP cert and key are not required"""
    entity_id = 'urn:mace:incommon:washington.edu'
    sso_url = 'https://idp.u.washington.edu/idp/profile/SAML2/Redirect/SSO'
    x509_cert = '''
        MIID/TCCAuWgAwIBAgIJAMoYJbDt9lKKMA0GCSqGSIb3DQEBBQUAMFwxCzAJBgNV
        BAYTAlVTMQswCQYDVQQIEwJXQTEhMB8GA1UEChMYVW5pdmVyc2l0eSBvZiBXYXNo
        aW5ndG9uMR0wGwYDVQQDExRpZHAudS53YXNoaW5ndG9uLmVkdTAeFw0xMTA0MjYx
        OTEwMzlaFw0yMTA0MjMxOTEwMzlaMFwxCzAJBgNVBAYTAlVTMQswCQYDVQQIEwJX
        QTEhMB8GA1UEChMYVW5pdmVyc2l0eSBvZiBXYXNoaW5ndG9uMR0wGwYDVQQDExRp
        ZHAudS53YXNoaW5ndG9uLmVkdTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
        ggEBAMH9G8m68L0Hf9bmf4/7c+ERxgDQrbq50NfSi2YTQWc1veUIPYbZy1agSNuc
        4dwn3RtC0uOQbdNTYUAiVTcYgaYceJVB7syWf9QyGIrglZPMu98c5hWb7vqwvs6d
        3s2Sm7tBib2v6xQDDiZ4KJxpdAvsoPQlmGdgpFfmAsiYrnYFXLTHgbgCc/YhV8lu
        bTakUdI3bMYWfh9dkj+DVGUmt2gLtQUzbuH8EU44vnXgrQYSXNQkmRcyoE3rj4Rh
        hbu/p5D3P+nuOukLYFOLRaNeiiGyTu3P7gtc/dy/UjUrf+pH75UUU7Lb369dGEfZ
        wvVtITXsdyp0pBfun4CP808H9N0CAwEAAaOBwTCBvjAdBgNVHQ4EFgQUP5smx3ZY
        KODMkDglkTbduvLcGYAwgY4GA1UdIwSBhjCBg4AUP5smx3ZYKODMkDglkTbduvLc
        GYChYKReMFwxCzAJBgNVBAYTAlVTMQswCQYDVQQIEwJXQTEhMB8GA1UEChMYVW5p
        dmVyc2l0eSBvZiBXYXNoaW5ndG9uMR0wGwYDVQQDExRpZHAudS53YXNoaW5ndG9u
        LmVkdYIJAMoYJbDt9lKKMAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEFBQADggEB
        AEo7c2CNHEI+Fvz5DhwumU+WHXqwSOK47MxXwNJVpFQ9GPR2ZGDAq6hzLJLAVWcY
        4kB3ECDkRtysAWSFHm1roOU7xsU9f0C17QokoXfLNC0d7KoivPM6ctl8aRftU5mo
        yFJkkJX3qSExXrl053uxTOQVPms4ypkYv1A/FBZWgSC8eNoYnBnv1Mhy4m8bfeEN
        7qT9rFoxh4cVjMH1Ykq7JWyFXLEB4ifzH4KHyplt5Ryv61eh6J1YPFa2RurVTyGp
        HJZeOLUIBvJu15GzcexuDDXe0kg7sHD6PbK0xzEF/QeXP/hXzMxR9kQXB/IR/b2k
        4ien+EM3eY/ueBcTZ95dgVM='''
    attribute_map = {
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.6': 'eppn',
        'urn:oid:0.9.2342.19200300.100.1.1': 'uwnetid',
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.1': attribute.List('affiliations'),
        'urn:oid:1.3.6.1.4.1.5923.1.5.1.1': attribute.UWGroups('groups'),
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.9': attribute.List(
            'scoped_affiliations')
    }


class UwIdpTwoFactor(UwIdp):
    two_factor = True



class IdpConfig(object):
    """
    Idp base configuration with a common set of defaults that get
    overridden by subclasses.
    """
    entity_id = ''
    sso_url = ''
    sso_binding = 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'
    x509_cert = ''
    id_attribute = 'urn:oid:1.3.6.1.4.1.5923.1.1.1.6'  # eppn
    mapped_id_attribute = 'remote_user'
    attribute_map = {}



import urllib.parse


class SamlAuthenticator:
    def __init__(self, request, old_settings, *args, **kwargs):
        self.request = request
        self.idp = old_settings['idp']['entityId']
        self.errors = []

    def login(self, **kwargs):
        kwargs['idp'] = self.idp
        qs = urllib.parse.urlencode(kwargs)
        return f'mock-login?{qs}'

    def process_response(self):
        idp = self.request['post_data'].get('idp')
        if idp != self.idp:
            self.errors.append(f'idp mismatch {idp} != {self.idp}')
        return

    def get_attributes(self):
        """Just reflect what's posted right back."""
        items = self.request['post_data'].items()
        return dict((key, [value]) for key, value in items)

    def get_errors(self):
        return self.errors

    def get_last_error_reason(self):
        return self.errors[0]

    def get_last_authn_contexts(self):
        return []



import os
from setuptools import setup, find_packages

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
VERSION_FILE = os.path.join(BASE_DIR, 'uw_saml2', 'VERSION')
with open(VERSION_FILE) as f:
    VERSION = f.readlines()[-1].strip()

with open(os.path.join(BASE_DIR, 'README.md')) as f:
    LONG_DESCRIPTION = f.read()

saml_requires = ['python3-saml']
tests_require = saml_requires + ['pytest', 'pytest-cov', 'mock', 'pycodestyle']


setup(name='uw-saml',
      version=VERSION,
      url='https://github.com/UWIT-IAM/uw-saml-python',
      author='UW-IT Identity and Access Management',
      author_email='help@uw.edu',
      description='A UW-specific adapter to the python3-saml package.',
      long_description=LONG_DESCRIPTION,
      long_description_content_type='text/markdown',
      license='Apache License, Version 2.0',
      packages=find_packages(),
      include_package_data=True,
      extras_require={'python3-saml': saml_requires, 'test': tests_require}
      )
